<?php

namespace Drupal\Pardot\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Executable\ExecutableManagerInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\salesforce\SalesforceAuthProviderPluginManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Pardot settings for this site.
 */
class PardotSettingsForm extends ConfigFormBase {

  /**
   * Configuration settings.
   */
  protected $settings;

  /**
   * @var \Drupal\system\Plugin\Condition\RequestPath $path_condition .
   */
  protected $path_condition;

  /**
   * @var \Drupal\user\Plugin\Condition\UserRole $user_role_condition .
   */
  protected $user_role_condition;

  /**
   * Auth provider manager.
   *
   * @var \Drupal\salesforce\SalesforceAuthProviderPluginManagerInterface
   */
  protected $authManager;

  /**
   * PardotSettingsForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Executable\ExecutableManagerInterface $plugin_factory
   *   The condition plugin factory.
   * @param \Drupal\salesforce\SalesforceAuthProviderPluginManagerInterface $authManager
   *   Auth manager service.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */

  public function __construct(ConfigFactoryInterface $config_factory, ExecutableManagerInterface $plugin_factory, SalesforceAuthProviderPluginManagerInterface $authManager) {
    parent::__construct($config_factory);
    // Load from pardot.settings.yml.
    $this->settings = $this->config('pardot.settings');

    // Create condition plugins.
    $this->path_condition = $plugin_factory->createInstance('request_path');
    $this->user_role_condition = $plugin_factory->createInstance('user_role');

    $this->authManager = $authManager;
    $this->authProviders = $authManager->getProviders();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.condition'),
      $container->get('plugin.manager.salesforce.auth_providers'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'pardot_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['pardot.settings'];
  }

  /**
   * Build Pardot Settings form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['pardot_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('General Settings'),
      '#description' => $this->t('General settings applicable to all Pardot functionality.'),
      '#open' => TRUE,
    ];

    $form['pardot_settings']['account_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Pardot Account ID'),
      '#description' => $this->t('The value shown in the Pardot demo script for piAId. eg. if the script has piAId = "1001"; this field should be 1001.'),
      '#required' => TRUE,
      '#size' => 20,
      '#maxlength' => 64,
      '#default_value' => $this->settings->get('account_id'),
    ];

    $form['pardot_settings']['default_campaign_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default Pardot Campaign ID'),
      '#description' => $this->t('The value shown in the Pardot demo script for piCId. eg. if the script has piCId = "1001"; this field should be 1001.'),
      '#required' => TRUE,
      '#size' => 20,
      '#maxlength' => 64,
      '#default_value' => $this->settings->get('default_campaign_id'),
    ];

    $form['api_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('API Settings'),
      '#description' => $this->t('Credentials, required to authenticate into the API. If not provided, all API-related functionality will be disabled'),
      '#open' => TRUE,
    ];

    $form['api_settings']['api_url'] = [
      '#type' => 'url',
      '#title' => $this->t('Pardot Environment Url'),
      '#description' => $this->t('Pardot Environment Url, without a trailing slash. See @link for more details', [
        '@link' => new TranslatableMarkup('<a href="https://developer.pardot.com/#test-and-production-environments">documentation</a>'),
      ]),
      '#size' => 20,
      '#require' => TRUE,
      '#default_value' => $this->settings->get('api_url') ?? 'https://pi.pardot.com',
    ];

    $options = [
      '' => 'None',
    ];

    // build up the select options
    foreach ($this->authProviders as $auth_provider_id => $auth_provider) {
      $options[$auth_provider_id] = $auth_provider->label();
    }

    $form['api_settings']['salesforce_auth_provider'] = [
      '#type' => 'select',
      '#required' => TRUE,
      '#title' => $this->t('Salesforce Auth Provider'),
      '#description' => $this->t('Select the Salesforce Auth Provider you would like to use to authenticate requests with.'),
      '#default_value' => $this->settings->get('salesforce_auth_provider') ?? '',
      '#options' => $options,
    ];

    $form['api_settings']['salesforce_pardot_business_unit_id'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Salesforce Business Unit ID'),
      '#description' => $this->t('Enter you salesforce Business Unit ID.'),
      '#default_value' => $this->settings->get('salesforce_pardot_business_unit_id') ?? '',
    ];

    // Add tracking scope vertical tabs.
    $form['tracking_scope'] = [
      '#type' => 'item',
      '#title' => $this->t('Tracking scope'),
      '#description' => $this->t('Configuration to include/exclude the Pardot tracking code. The tracking code, with default Campaign ID, will be added to the site paths according to this configuration. Additional campaigns can be added to override the default campaign on specific paths included within this configuration.'),
    ];

    $form['tracking_scope_tabs'] = [
      '#type' => 'vertical_tabs',
      '#title' => $this->t('Tracking Scope Conditions'),
      '#title_display' => 'invisible',
      '#default_tab' => 'pages',
    ];

    // Set and build the request_path condition configuration form elements.
    $this->path_condition->setConfiguration($this->settings->get('path_condition'));
    $form['path_condition'] = $this->path_condition->buildConfigurationForm([], $form_state);
    $form['path_condition']['#type'] = 'details';
    $form['path_condition']['#tree'] = TRUE;
    $form['path_condition']['#group'] = 'tracking_scope_tabs';
    $form['path_condition']['#title'] = $this->t('Pages');
    $form['path_condition']['negate']['#type'] = 'radios';
    $form['path_condition']['negate']['#title_display'] = 'invisible';
    $form['path_condition']['negate']['#options'] = [
      $this->t('Show for the listed pages'),
      $this->t('Hide for the listed pages'),
    ];
    $form['path_condition']['negate']['#default_value'] = (int) $form['path_condition']['negate']['#default_value'];

    // Set and build the user_role condition configuration form elements.
    $this->user_role_condition->setConfiguration($this->settings->get('user_role_condition'));
    $form['user_role_condition'] = $this->user_role_condition->buildConfigurationForm([], $form_state);
    $form['user_role_condition']['#type'] = 'details';
    $form['user_role_condition']['#tree'] = TRUE;
    $form['user_role_condition']['#group'] = 'tracking_scope_tabs';
    $form['user_role_condition']['#title'] = $this->t('Roles');
    $form['user_role_condition']['negate']['#type'] = 'value';
    $form['user_role_condition']['negate']['#value'] = $form['user_role_condition']['negate']['#default_value'];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // @Todo: replace custom validation with plugin validation, if core
    // issue gets included: https://www.drupal.org/node/2843992.
    // $this->path_condition->validateConfigurationForm($form, $form_state);

    // Cast negate values to boolean instead of strings.
    $path_condition_value = $form_state->getValue('path_condition');
    if(array_key_exists('negate', $path_condition_value)) {
      $form_state->setValue(['path_condition', 'negate'], (bool) $path_condition_value['negate']);
    }
    $user_role_condition = $form_state->getValue('user_role_condition');
    if(array_key_exists('negate', $user_role_condition)) {
      $form_state->setValue(['user_role_condition', 'negate'], (bool) $user_role_condition['negate']);
    }

    // Validates path conditions to ensure they have leading forward slash.
    $paths = explode("\r\n", $form_state->getValue('pages'));
    foreach ($paths as $path) {
      if (empty($path) || $path === '<front>' || strpos($path, '/') === 0) {
        continue;
      }
      else {
        $message = $this->t('Paths require a leading forward slash when used with the Tracking Scope Pages setting.');
        $form_state->setErrorByName('pages', $message);
        return;
      }
    }

    //    $salesforce_auth_provider_value = $form_state->getValue('salesforce_auth_provider');
    /** @var \Drupal\salesforce\SalesforceAuthProviderPluginManager $auth_provider_service */
    //    $auth_provider_service = \Drupal::service('plugin.manager.salesforce.auth_providers');
    //    $auth_providers = $auth_provider_service->getProviders();
    //    $salesforce_auth_provider = $auth_providers[$salesforce_auth_provider_value] ?? false;
    //    if (!is_object($salesforce_auth_provider) && $salesforce_auth_provider instanceof \Drupal\salesforce\Entity\SalesforceAuthConfig){
    //      $message = $this->t('Please choose a valid salesforse auth provider.');
    //      $form_state->setErrorByName('salesforce_auth_provider', $message);
    //      return;
    //    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Submit condition plugin configurations.
    $this->path_condition->submitConfigurationForm($form['path_condition'],  SubformState::createForSubform($form['path_condition'], $form, $form_state));
    $this->user_role_condition->submitConfigurationForm($form['user_role_condition'],  SubformState::createForSubform($form['user_role_condition'], $form, $form_state));

    // Save configuration to settings.
    $this->config('pardot.settings')
      ->set('account_id', Html::escape($form_state->getValue('account_id')))
      ->set('default_campaign_id', Html::escape($form_state->getValue('default_campaign_id')))
      ->set('api_url', $form_state->getValue('api_url'))
      ->set('salesforce_auth_provider', $form_state->getValue('salesforce_auth_provider'))
      ->set('salesforce_pardot_business_unit_id', $form_state->getValue('salesforce_pardot_business_unit_id'))
      ->set('path_condition', $this->path_condition->getConfiguration())
      ->set('user_role_condition', $this->user_role_condition->getConfiguration())
      ->save();

    parent::submitForm($form, $form_state);
  }

}
