<?php

namespace Drupal\pardot;

/**
 * Provides an interface for defining Pardot Form Map entities.
 */
class FormMap implements FormMapInterface {

  /**
   * The post url provided by the pardot form handler.
   *
   * @var string
   */
  protected $post_url;

  /**
   * The id of the form handler.
   *
   * @var id
   */
  protected $form_handler_id;

  /**
   * The array of mapped contact form fields to pardot form handler fields.
   *
   * @var array
   */
  public $settings = [];

  /**
   * The array of mapped contact form fields to pardot form handler fields.
   *
   * @var array
   */
  public $mapping = [];

  /**
   * {@inheritdoc}
   */
  public function __construct($settings) {
    $mapping = [];
    $this->settings = $settings;
    $form_map_plugin_manager = \Drupal::service('plugin.manager.pardot_field_formatter');
    if (isset($this->settings['form_map'])) {
      foreach ($this->settings['form_map'] as $key => $setting) {
        if ($setting['plugin_type']) {
          $plugin = $form_map_plugin_manager->createInstance($setting['plugin']['id']);
          $plugin->setConfiguration($setting['plugin']);
        }
        else {
          $plugin = [];
        }
        $class = $setting['class'];
        $mapping[$key] = new $class($setting['pardot_key'], $setting['plugin_type'], $plugin);
      }
    }

    $post_url = $this->settings['post_url'] ?? '';
    $form_handler_id = $this->settings['form_handler_id'] ?? '';

    $this->setPostUrl($post_url);
    $this->setFormHandlerId(intval($form_handler_id));
    $this->setMappedFieldCollection($mapping);
  }

  /**
   * {@inheritdoc}
   */
  public function getPostUrl() {
    return trim($this->post_url);
  }

  /**
   * {@inheritdoc}
   */
  public function setPostUrl(string $post_url) {
    $this->post_url = trim($post_url);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormHandlerId() {
    return $this->form_handler_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setFormHandlerId(string $id) {
    $this->form_handler_id = $id;
  }

  /**
   * {@inheritdoc}
   */
  public function getMappedFieldCollection() {
    return $this->mapping;
  }

  /**
   * {@inheritdoc}
   */
  public function setMappedFieldCollection(array $mapping) {
    $this->mapping = $mapping;
  }

  /**
   * Add a new MappedField instance to the collection array.
   *
   * @param string $class
   *   The specific class name.
   * @param string $pardot_key
   *   The name of the pardot field.
   * @param string $plugin_type
   *   The machine name of the selected plugin.
   * @param mixed $plugin
   *   The plugin instance or empty array.
   * @param array $config
   *   Pass some optional init config to the plugin.
   * @param bool $required
   *   Is this field required to return a value.
   */
  public function appendMappedField(string $class = MappedField::class, string $pardot_key = '', string $plugin_type = '', $plugin = [], array $config = [], $required = FALSE) {
    $this->mapping[] = new $class($pardot_key, $plugin_type, $plugin, $config, $required);
  }

  /**
   * Retrieve a particular row from the MappedField collection.
   *
   * @param int $key
   *   The index where the MappedField lives.
   *
   * @return mixed
   *   Return the instance of MappedFieldInterface or null.
   */
  public function getMappedField(int $key) {
    if (isset($this->mapping[$key])) {
      return $this->mapping[$key];
    }
    return NULL;
  }

  /**
   * Remove a particular row from the MappedField collection.
   *
   * @param int $key
   *   The index where the instance of MappedFieldInterface lives.
   */
  public function removeMappedField(int $key) {
    if (isset($this->mapping[$key])) {
      unset($this->mapping[$key]);
    }
  }


  /**
   * Set a field plugin status to FALSE.
   *
   * @param int $key
   *   The index where the instance of MappedFieldInterface lives.
   */
  public function disableMappedField(int $key) {
    if (isset($this->mapping[$key])) {
      /** @var \Drupal\pardot\MappedFieldInterface $plugin */
      $plugin = $this->mapping[$key];
      $plugin->setStatus(FALSE);
    }
  }

  /**
   * Set a field plugin status to TRUE.
   *
   * @param int $key
   *   The index where the instance of MappedFieldInterface lives.
   */
  public function enableMappedField(int $key) {
    if (isset($this->mapping[$key])) {
      /** @var \Drupal\pardot\MappedFieldInterface $plugin */
      $plugin = $this->mapping[$key];
      $plugin->setStatus(TRUE);
    }
  }

  /**
   * Set a field plugin status to FALSE.
   *
   * @param int $key
   *   The index where the instance of MappedFieldInterface lives.
   */
  public function requireMappedField(int $key) {
    if (isset($this->mapping[$key])) {
      /** @var \Drupal\pardot\MappedFieldInterface $plugin */
      $plugin = $this->mapping[$key];
      $plugin->setIsRequired(TRUE);
    }
  }


  public function toArray() {
    $mapping = $this->mapping;
    $mapping = array_map(function (MappedFieldInterface $mapping) {
      return $mapping->toArray();
    }, $mapping);

    $this->settings['post_url'] = $this->getPostUrl();
    $this->settings['form_handler_id'] = $this->getFormHandlerId();
    $this->settings['form_map'] = $mapping;
    return $this->settings;
  }

}
