<?php

namespace Drupal\pardot;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\EntityFieldManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\pardot\Plugin\PardotFieldFormatterManager;

/**
 * Manipulates entity type information.
 *
 * This class contains primarily bridged hooks for compile-time or
 * cache-clear-time hooks. Runtime hooks should be placed in EntityOperations.
 */
class FormMapForm {
  use StringTranslationTrait;
  /**
   * Array of settings to build with.
   *
   * @var array
   */
  protected $configuration = [];

  /**
   * Entity field manager to get the contact form field definitions.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected EntityFieldManager $entityFieldManager;

  /**
   * Plugin factory to create instances of plugins for formatters.
   *
   * @var \Drupal\pardot\Plugin\PardotFieldFormatterManager
   */
  protected PardotFieldFormatterManager $pardotFormMapFormatterManagerPlugin;

  /**
   * EntityTypeInfo constructor.
   *
   * @param \Drupal\pardot\Plugin\PardotFieldFormatterManager $pardotFormMapFormatterManagerPlugin
   *   The formatter plugins manager.
   * @param \Drupal\Core\Entity\EntityFieldManager $entityFieldManager
   *   The entity field manager for grabbing fields off entities.
   */
  public function __construct(PardotFieldFormatterManager $pardotFormMapFormatterManagerPlugin, EntityFieldManager $entityFieldManager, array $configuration) {
    $this->pardotFormMapFormatterManagerPlugin = $pardotFormMapFormatterManagerPlugin;
    $this->entityFieldManager = $entityFieldManager;
    $this->configuration = $configuration;
  }

  public static function createInstance(array $configuration = []) {
    $container = \Drupal::getContainer();
    return new static(
      $container->get('plugin.manager.pardot_field_formatter'),
      $container->get('entity_field.manager'),
      $configuration
    );
  }

  public function buildForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\pardot\FormMap $pardot_form_map */
    if (empty($form_state->get('pardot_form_map'))) {
      $pardot_form_map = new FormMap($this->configuration);
      $form_state->set('pardot_form_map', $pardot_form_map);
    }
    else {
      $pardot_form_map = $form_state->get('pardot_form_map');
    }

    $form['post_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Post Url'),
      '#required' => TRUE,
      '#description' => $this->t('Enter the form handler post url from Pardot.'),
      '#default_value' => $pardot_form_map->getPostUrl(),
    ];

    $form['settings_container'] = [
      '#type' => 'container',
      //      '#attributes' => ['id' => 'pardot-settings-wrapper'],
      '#prefix' => '<div id="pardot-settings-wrapper">',
      '#suffix' => '</div>',
    ];

    $header = [
      'pardot_key' => [
        'data' => $this->t('Pardot Key'),
      ],
      'field' => [
        'data' => $this->t('Field Format'),
      ],
      'remove' => [
        'data' => $this->t('Remove'),
      ],
    ];

    $form['settings_container']['settings'] = [
      '#type' => 'table',
      '#caption' => $this->t('Add Pardot External Field Names to map contact form elements.'),
      '#header' => $header,
      '#tree' => TRUE,
    ];

    if (!$pardot_form_map->getMappedFieldCollection()) {
      $this->injectRequiredFields($pardot_form_map);
    }

    $mapping = $pardot_form_map->getMappedFieldCollection();

    // Generate tables rows.
    if ($mapping) {
      foreach ($mapping as $key => $value) {
        $this->buildRow($form, $form_state, $key, $value);
      }
    }

    $form['add_field_map'] = [
      '#type' => 'submit',
      '#name' => 'add_field_map',
      '#value' => $this->t('Add Field Map'),
      '#depth' => -1,
      '#submit' => [[$this, 'addFieldMapSubmit']],
      '#limit_validation_errors' => [],
      '#ajax' => [
        'callback' => [$this, 'ajaxAddFieldMap'],
        'wrapper' => 'pardot-settings-wrapper',
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function injectRequiredFields(FormMap $pardot_form_map) {
  }

  /**
   * Build the form map row.
   *
   * @param array $form
   *   The form for creating the subform.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state to use for creating the subform state.
   * @param int $key
   *   Row index.
   * @param \Drupal\pardot\MappedFieldInterface $row
   *   Row data.
   */
  public function buildRow(array &$form, FormStateInterface $form_state, int $key, MappedFieldInterface $row) {
    $plugin_collection = $this->getPluginCollection();

    $plugin_type_submit_button = $this->generatePluginTypeSubmitButton($key);

    $plugin_form = PluginSubFormHelpers::buildPluginForm($row->getPlugin(), $form, $form_state, [
      'settings_container',
      'settings',
      $key,
      'formatter_settings',
      'plugin',
    ]);

    $description = ($row instanceof MappedEmailField) ? 'Enter the email field name. This field is required by Pardot. This will also be the email used to pair a user with tracking info.' : 'Enter the field name from the pardot form handler.';
    $form['settings_container']['settings'][$key] = [
      'pardot_key' => [
        '#type' => 'textfield',
        '#title' => $this->t('Pardot Field Name'),
        '#description' => $this->t($description),
        '#title_display' => 'invisible',
        '#required' => TRUE,
        '#default_value' => $row->getPardotKey(),
      ],
      'formatter_settings' => [
        '#type' => 'container',
        'plugin_type' => [
          '#type' => 'select',
          '#title' => $this->t('Formatter Selection'),
          '#access' => !($row instanceof MappedEmailField),
          '#title_display' => 'invisible',
          '#default_value' => $row->getPluginType(),
          '#options' => $plugin_collection,
          '#ajax' => [
            'event' => 'change',
            'callback' => [$this, 'updateFieldMap'],
            'wrapper' => 'pardot-settings-wrapper',
            'trigger_as' => ['name' => "settings_$key"],
          ],
        ],
        'plugin' => $plugin_form,
        'submit' => $plugin_type_submit_button,
      ],
      'remove' => [
        '#type' => 'submit',
        '#name' => 'remove_field_map_' . $key,
        '#value' => $this->t('Remove'),
        '#submit' => [[$this, 'removeFieldMapSubmit']],
        '#depth' => -4,
        '#limit_validation_errors' => [],
        '#ajax' => [
          'callback' => [$this, 'ajaxRemoveFieldMap'],
          'wrapper' => 'pardot-settings-wrapper',
        ],
      ],
    ];
  }

  /**
   * Return a submit button for on change field selection.
   *
   * @param int $key
   *   The row key.
   *
   * @return array
   *   The submit button for the row.
   */
  public function generatePluginTypeSubmitButton($key) {
    return [
      '#submit' => [[$this, 'pluginTypeChangeSubmit']],
      '#type' => 'submit',
      '#attributes' => ['class' => ['visually-hidden']],
      '#ajax' => [
        'callback' => [$this, 'ajaxChangeFieldMap'],
        'wrapper' => 'pardot-settings-wrapper',
        'effect' => 'fade',
      ],
      '#limit_validation_errors' => [],
      '#name' => "settings_$key",
      '#row_index' => $key,
    ];
  }

  /**
   * Given a trigger element and depth return the correct parents for the form
   * area to reload on ajax.
   *
   * @param array $trigger_element
   *   The triggering element from the ajax call.
   * @param int $depth
   *   The depth to cut off the end to get to the root of the form.
   *
   * @return array
   *   Return the correct parents array for the ajax part of the form.
   */
  public function formParentsArray(array $trigger_element, int $depth) {
    $array_parents = array_slice($trigger_element['#array_parents'], 0, $depth);
    array_push($array_parents, 'settings_container');
    return $array_parents;
  }

  /**
   * This is used to update the dom of the table container when adding a row.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return array|mixed|null
   *   Just return the mapped field container.
   */
  public function ajaxAddFieldMap(array &$form, FormStateInterface $form_state) {
    $trigger_element = $form_state->getTriggeringElement();
    return NestedArray::getValue($form, $this->formParentsArray($trigger_element, -1));
  }

  /**
   * Add a new form field mapping row.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function addFieldMapSubmit(array &$form, FormStateInterface $form_state) {
    $pardot_form_map = $form_state->get('pardot_form_map');
    $pardot_form_map->appendMappedField();
    $form_state->set('pardot_form_map', $pardot_form_map);
    $form_state->setRebuild();
  }

  /**
   * This is used to update the dom of the table container when changing a row
   * plugin.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return array|mixed|null
   *   Just return the mapped field container.
   */
  public function ajaxChangeFieldMap(array &$form, FormStateInterface $form_state) {
    $trigger_element = $form_state->getTriggeringElement();
    return NestedArray::getValue($form, $this->formParentsArray($trigger_element, -5));
  }

  /**
   * Set the mapping array on change.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function pluginTypeChangeSubmit(array &$form, FormStateInterface $form_state) {
    $trigger_element = $form_state->getTriggeringElement();
    $parents = array_slice($trigger_element['#array_parents'], 0, -1);
    $parents[] = 'plugin_type';
    $key = $trigger_element['#row_index'];
    $plugin_type = NestedArray::getValue($form, $parents)['#value'];
    $pardot_form_map = $form_state->get('pardot_form_map');
    // @todo prolly need to check if getMappedField is null.
    $mapped_field = $pardot_form_map->getMappedField($key);
    $mapped_field->setPluginType($plugin_type);
    if ($mapped_field && !($mapped_field instanceof MappedEmailField)) {
      if ($plugin_type) {
        $plugin = $this->pardotFormMapFormatterManagerPlugin->createInstance($plugin_type)
          ->setConfiguration([
            'entity_types' => $this->configuration['entity_types'],
            'entity_id' => $this->configuration['entity_type_id'] . ':' . $this->configuration['entity_id'],
          ]);
        $pardot_form_map->getMappedField($key)->setPlugin($plugin);
      }
      else {
        $pardot_form_map->getMappedField($key)->setPlugin([]);
      }
    }
    $form_state->set('pardot_form_map', $pardot_form_map);
    $form_state->setRebuild();
  }

  /**
   * This is used to update the dom of the table container when removing a row.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return array|mixed|null
   *   Just return the mapped field container.
   */
  public function ajaxRemoveFieldMap(array &$form, FormStateInterface $form_state) {
    $trigger_element = $form_state->getTriggeringElement();
    return NestedArray::getValue($form, $this->formParentsArray($trigger_element, -4));
  }

  /**
   * Remove specified row from mapping.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function removeFieldMapSubmit(array &$form, FormStateInterface $form_state) {
    $pardot_form_map = $form_state->get('pardot_form_map');
    $trigger_element = $form_state->getTriggeringElement();
    $parents = array_slice($trigger_element['#parents'], 0, -1);
    $map_index = array_pop($parents);
    $mappedRow = $pardot_form_map->getMappedField($map_index);
    if ($mappedRow && !($mappedRow instanceof MappedEmailField)) {
      $pardot_form_map->removeMappedField($map_index);
      $form_state->set('pardot_form_map', $pardot_form_map);
    }
    $form_state->setRebuild();
  }

  public function getPluginCollection() {
    $plugins = $this->pardotFormMapFormatterManagerPlugin->getDefinitions();
    $options = ['' => 'none'];
    foreach ($plugins as $plugin_name => $plugin_definition) {
      if (isset($plugin_definition['types']) && in_array($this->configuration['entity_type_id'], $plugin_definition['types'])) {
        $options[$plugin_name] = $plugin_definition['label'];
      }
    }
    return $options;
  }

}
