<?php


namespace Drupal\pardot;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\State\State;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\pardot\Service\PardotClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Stuff.
 *
 *
 * @internal
 */
class FormOperations implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * State manager.
   *
   * @var \Drupal\Core\State\State
   */
  protected State $state;

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private ImmutableConfig $pardotSettings;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  private LoggerChannelFactoryInterface $logger;

  /**
   * @var \Drupal\pardot\Service\PardotClientInterface
   */
  private PardotClientInterface $pardotClient;


  /**
   * Stuff.
   *
   * @param \Drupal\Core\State\State $state
   *   The core state.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerChannelFactory
   *   Channel for logging messages.
   * @param \Drupal\pardot\Service\PardotClientInterface $pardotClient
   *   Client to interact with salesforce/pardot endpoints.
   */
  public function __construct(State $state, ConfigFactoryInterface $configFactory, EntityTypeManagerInterface $entityTypeManager, LoggerChannelFactoryInterface $loggerChannelFactory, PardotClientInterface $pardotClient) {
    $this->state = $state;
    $this->pardotSettings = $configFactory->get('pardot.settings');
    $this->entityTypeManager = $entityTypeManager;
    $this->logger = $loggerChannelFactory;
    $this->pardotClient = $pardotClient;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('state'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('logger.factory'),
      $container->get('pardot.api_client'),
    );
  }

  /**
   * Get the respective form entity.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param string $formClassName
   *   The Class string to check the form objects against.
   * @param string $entity_type_id
   *   The entity type id we should get the storage for.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getFormEntity(array &$form, FormStateInterface $form_state, string $formClassName, string $entity_type_id): void {
    // Get conditional state variable.
    // @see \Drupal\pardot\EventSubscriber\PardotEventSubscriber
    $include_tracking = (bool) $this->state
      ->get('pardot.include_tracking') ?: 0;

    if ($include_tracking && NULL !== $this->pardotSettings->get('account_id')) {
      $bundle = FALSE;
      $form_object = $form_state->getFormObject();
      if ($form_object instanceof $formClassName) {
        $bundle = $form_object->getEntity()->bundle();
        $entity = $this->entityTypeManager->getStorage($entity_type_id)->load($bundle);
        if ($entity) {
          $this->setPardotFormSubmit($entity, $form, $form_state);
        }
      }
    }
  }

  /**
   * Set Pardot form submit.
   *
   * @param object $entity
   *   The entity (form) you want to attach the pardot form submit functionality.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state interface.
   *
   * @see pardot_form_contact_message_form_alter()
   */
 public function setPardotFormSubmit(object $entity, &$form ,FormStateInterface $form_state): void {
    $pardot_form_map_plugin = $entity->getThirdPartySetting('pardot', 'pardot_form_mapping_plugin', FALSE);
    if ($pardot_form_map_plugin) {
      $pardot_form_map = new FormMap($pardot_form_map_plugin['settings']);
      if ($pardot_form_map) {
        $storage = $form_state->getStorage();
        $storage['pardot_form_map'] = $pardot_form_map;
        // Add submit handler if we have an enabled contact form mapping.
        $form_state->setStorage($storage);
        $form['actions']['submit']['#submit'][] = [$this, 'pardotFormSubmit'];
      }
    }
  }

  /**
   * Pardot contact form submit.
   *
   * @param array $form
   *   Form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state interface.
   *
   * @see pardot_form_contact_message_form_alter()
   */
  public function pardotFormSubmit(array &$form, FormStateInterface $form_state) {
    // Retrieve storage, where pardot_mapping was set in
    // pardot_form_contact_message_form_alter().
    $logger = $this->logger->get('pardot');
    $storage = $form_state->getStorage();
    if (isset($storage['pardot_form_map'])) {


      /** @var \Drupal\pardot\FormMap $pardot_form_map */
      $pardot_form_map = $storage['pardot_form_map'];
      $post_url = $pardot_form_map->getPostUrl();
      $field_map_collection = $pardot_form_map->getMappedFieldCollection();
      $visitor_email = '';
      $visitor_id = $this->pardotClient->getVisitorId();

      $query_params = [];

      // Loop thru field map and grab form values from submission.
      // Then build up the query params for the pardot submission.
      foreach ($field_map_collection as $field_map) {
        $plugin = $field_map->getPlugin();
        if ($plugin && $plugin->getFormattedValue($form_state)) {
          $value = $plugin->getFormattedValue($form_state);
          $query_params[] = [$field_map->getPardotKey() => $value];
          if ($field_map instanceof MappedEmailField) {
            $visitor_email = $value;
          }
        }
      }

      // Unable to use \Drupal\Component\Utility\UrlHelper::buildQuery() b/c
      // Pardot accepts multiple values using the same parameter key.
      $query_string = '';

      foreach ($query_params as $param) {
        $keys = array_keys($param);
        $key = reset($keys);
        $query_string .= rawurlencode($key) . '=' . str_replace('%2F', '/', rawurlencode($param[$key])) . '&';
      }

      $request_data = [
        'timeout' => 5,
        'query' => $query_string,
      ];

      $result = $this->pardotClient->executePardotOperation($post_url, $request_data);

      if ($result) {
        $logger->notice('Successful submission to the pardot form handler.');
      } else {
        $logger->error('Failed submission to the pardot form handler.');
      }

      // Connect the visitor ID to a prospect ID using the submitted email.
      if ($visitor_email && $visitor_id) {
        // Look up the prospect ID.
        $prospect_id = $this->pardotClient->getPardotProspect($visitor_email);

        // Use the visitor ID to update the prospect.
        $prospect_is_assigned = $this->pardotClient->assignPardotProspect($visitor_id, $prospect_id);
        if (!$prospect_is_assigned) {
          $logger->error('Unable to assign Pardot prospect. Failure returned from API request.');
        }
      }

    }
  }
}

