<?php

namespace Drupal\pardot;

/**
 * Provides a class for mapped fields.
 */
interface MappedFieldInterface {

  /**
   * Constructs a new MappedField.
   *
   * @param string $pardot_key
   *   The pardot field key to map the value to.
   * @param string $plugin_type
   *   The plugin id.
   * @param mixed $plugin
   *   (optional) The Plugin associated with this map.
   * @param mixed $config
   *   (optional) Pass some initial config to the plugin.
   */
  public function __construct(string $pardot_key = '', string $plugin_type = '', $plugin = [], $config = []);

  /**
   * Get the pardot key string.
   *
   * @return string
   *   Return the string for the pardot key.
   */
  public function getPardotKey();

  /**
   * Set the string for the pardot key.
   *
   * @param string $pardot_key
   *   The key for the pardot field.
   *
   * @return $this
   *   Return this instance to act on with chaining.
   */
  public function setPardotKey($pardot_key);

  /**
   * Set whether or not this plugin field is required.
   *
   * @param bool $bool
   *   Set the required prop to this bool.
   *
   * @return $this
   *   Return this instance to act on with chaining.
   */
  public function setIsRequired(bool $bool);

  /**
   * Return whether this plugin is disabled or not.
   *
   * @param bool $bool
   *   get the required prop to this bool.
   *
   * @return bool
   *   Return whether this plugin is required or not.
   */
  public function isRequired();

  /**
   * Set whether or not this plugin field is disabled.
   *
   * @param bool $bool
   *   Set the disabled prop to this bool.
   *
   * @return $this
   *   Return this instance to act on with chaining.
   */
  public function setStatus(bool $bool);

  /**
   * Return whether this plugin is disabled or not.
   *
   * @param bool $bool
   *   Set the disabled prop to this bool.
   *
   * @return bool
   *   Return whether this plugin is disabled or not.
   */
  public function getStatus();

  /**
   * Get the machine name of the option chosen for the plugin type.
   *
   * @return string
   *   The machine name of the field/option chosen.
   */
  public function getPluginType();

  /**
   * Set the machine name of the plugin_type chosen.
   *
   * @param string $plugin_type
   *   The machine name of the plugin_type chosen.
   *
   * @return $this
   *   Return this instance to act on with chaining.
   */
  public function setPluginType(string $plugin_type);

  /**
   * Return the plugin instance or null.
   *
   * @return mixed|null
   *   Return the plugin instance or null.
   */
  public function getPlugin();

  /**
   * Set the instance of the plugin for this MappedField.
   *
   * @param mixed $plugin
   *   The instance of a plugin or [].
   *
   * @return $this
   *   Return this instance to act on with chaining.
   */
  public function setPlugin($plugin);

  /**
   * Returns an array representation of the segment.
   *
   * @return array
   *   An array representation of the segment.
   */
  public function toArray();

}
