<?php

namespace Drupal\pardot\Plugin;

use Drupal\Component\Plugin\ConfigurableInterface;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Executable\ExecutableManagerInterface;
use Drupal\Core\Executable\ExecutablePluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContextAwarePluginAssignmentTrait;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\pardot\FormMap;
use Drupal\pardot\MappedEmailField;
use Drupal\pardot\MappedField;
use Drupal\pardot\MappedFieldInterface;
use Drupal\pardot\PluginSubFormHelpers;

/**
 * Base class for Form Map Handler Plugin plugins.
 */
abstract class PardotFormHandlerBase extends ExecutablePluginBase implements PardotFormHandlerInterface, PluginFormInterface, ConfigurableInterface {

  use ContextAwarePluginAssignmentTrait;

  /**
   * Entity field manager to get the contact form field definitions.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityFieldManager;

  /**
   * Plugin factory to create instances of plugins for formatters.
   *
   * @var \Drupal\pardot\Plugin\PardotFieldFormatterManager
   */
  protected $pardotFormMapFormatterManagerPlugin;

  /**
   * Pardot api client to interact with endpoints.
   *
   * @var \Drupal\pardot\Service\PardotClient
   */
  protected $pardotApiClient;

  /**
   * The condition manager to proxy execute calls through.
   *
   * @var \Drupal\Core\Executable\ExecutableManagerInterface
   */
  protected ExecutableManagerInterface $executableManager;

  /**
   * Class that defines a form mapping.
   *
   * @var \Drupal\pardot\FormMap
   */
  protected FormMap $formMap;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    // Not sure how else to instantiate these.
    $this->entityFieldManager = \Drupal::service('entity_field.manager');
    $this->pardotFormMapFormatterManagerPlugin = \Drupal::service('plugin.manager.pardot_field_formatter');
    $this->pardotApiClient = \Drupal::service('pardot.api_client');
    $this->setConfiguration($configuration);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'settings' => [],
      'entity_type_id' => '',
      'entity_id' => '',
      'entity_types' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return [
        'id' => $this->getPluginId(),
      ] + $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = $configuration + $this->defaultConfiguration();
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $form_values = $form_state->getValues();
    $form_handler_id = $form_values['pardot_form_mapping']['form_handler_id'] ?? FALSE;
    $check_valid_form_id = $this->pardotApiClient->getFormHandlerFields($form_handler_id);
    if (!$check_valid_form_id && $form_handler_id !== '-1') {
      // We want the literal %author placeholder to be emphasized in the error message.
      $form_state->setErrorByName('form_handler_id', t('Please enter a valid form handler id or a form handler with fields present.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function execute() {
    return $this->executableManager->execute($this);
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\pardot\FormMap $pardot_form_map */
    if (empty($form_state->get('pardot_form_map'))) {
      $pardot_form_map = new FormMap($this->configuration['settings']);
      $form_state->set('pardot_form_map', $pardot_form_map);
    }
    else {
      $pardot_form_map = $form_state->get('pardot_form_map');
    }

    $pardot_fields = $this->pardotApiClient->getFormHandlerFields($pardot_form_map->getFormHandlerId());

    $form['post_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Post Url'),
      '#required' => TRUE,
      '#description' => $this->t('Enter the form handler post url from Pardot.'),
      '#default_value' => $pardot_form_map->getPostUrl(),
    ];

    $form['form_handler_id'] = [
      '#type' => 'number',
      '#title' => $this->t('Form Handler Id'),
      '#required' => TRUE,
      '#min' => -1,
      '#description' => $this->t('Enter the form handler id from Pardot. (Can find in the url when viewing the handler (Example "/formHandler/read/id/394"). Use -1 to manually enter fields.'),
      '#default_value' => $pardot_form_map->getFormHandlerId(),
    ];

    $form['settings_container'] = [
      '#type' => 'container',
      //      '#attributes' => ['id' => 'pardot-settings-wrapper'],
      '#prefix' => '<div id="pardot-settings-wrapper">',
      '#suffix' => '</div>',
    ];

    $header = [
      'pardot_key' => [
        'data' => $this->t('Pardot Key'),
      ],
      'field' => [
        'data' => $this->t('Field Format'),
      ],
      'remove' => [
        'data' => $this->t('Remove'),
      ],
    ];
    $form['settings_container']['settings'] = [
      '#type' => 'table',
      '#caption' => $this->t('Add Pardot External Field Names to map contact form elements.'),
      '#header' => $header,
      '#tree' => TRUE,
      '#access' =>  ($pardot_form_map->getFormHandlerId() === '-1' || $pardot_fields),
    ];

    if ($pardot_fields) {
      $this->syncPardotFields($pardot_form_map, $pardot_fields);
    }

    $mapping = $pardot_form_map->getMappedFieldCollection();

    // Generate tables rows.
    if ($mapping) {
      foreach ($mapping as $key => $value) {
          $this->buildRow($form, $form_state, $key, $value);
      }
    }

    if ($pardot_form_map->getFormHandlerId() === '-1') {
      $form['add_field_map'] = [
        '#type' => 'submit',
        '#name' => 'add_field_map',
        '#value' => $this->t('Add Field Map'),
        '#depth' => -1,
        '#submit' => [[$this, 'addFieldMapSubmit']],
        '#limit_validation_errors' => [],
        '#ajax' => [
          'callback' => [$this, 'ajaxAddFieldMap'],
          'wrapper' => 'pardot-settings-wrapper',
        ],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function syncPardotFields(FormMap $pardot_form_map, $pardot_fields) {
    $pardot_form_map_array = $pardot_form_map->toArray();
    $pardot_form_map_array = $pardot_form_map_array['form_map'] ?? [];

    foreach ($pardot_form_map_array as $key => $map_item) {
      $is_not_in_pardot = !in_array($map_item['pardot_key'], array_column($pardot_fields, 'name'));
      if ($is_not_in_pardot) {
        $pardot_form_map->disableMappedField($key);
      } else {
        $pardot_form_map->enableMappedField($key);
      }
    }

    foreach ($pardot_fields as $pardot_field) {
      $is_field_mapped = in_array($pardot_field['name'], array_column($pardot_form_map_array, 'pardot_key'));
      if (!$is_field_mapped) {
        if ($pardot_field['dataFormat'] !== 'email') {
          $required = $pardot_field['isRequired'] ?? FALSE;
          $pardot_form_map->appendMappedField(MappedField::class, $pardot_field['name'], '', [], [], $required);
        } else {
          $plugin_definition = $this->getPluginDefinition();
          $email_plugin_type_id = $plugin_definition['email_field_formatter_id'] ?? '';
          $pardot_form_map->appendMappedField(MappedEmailField::class, $pardot_field['name'], $email_plugin_type_id, [], ['entity_id' => $this->configuration['entity_type_id'] . ':' . $this->configuration['entity_id']]);
        }
      }
    }
  }

  /**
   * Build the form map row.
   *
   * @param array $form
   *   The form for creating the subform.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state to use for creating the subform state.
   * @param int $key
   *   Row index.
   * @param \Drupal\pardot\MappedFieldInterface $row
   *   Row data.
   * @param string $form_handler_id
   *   Form handler id.
   */
  public function buildRow(array &$form, FormStateInterface $form_state, int $key, MappedFieldInterface $row) {
    $pardot_form_map = $form_state->get('pardot_form_map');
    $form_handler_id = $pardot_form_map->getFormHandlerId();
    $field_message = $form_handler_id !== '-1' && !$row->getStatus() ? 'This field name does not match any in the pardot form handler.' : '';
    $plugin_collection = $this->getPluginCollection();
    $plugin_type_submit_button = $this->generatePluginTypeSubmitButton($key);
    $description = ($row instanceof MappedEmailField) ? 'Enter the email field name. This field is required by Pardot. This will also be the email used to pair a user with tracking info.' : 'Enter the field name from the pardot form handler.';
    $plugin_form = PluginSubFormHelpers::buildPluginForm($row->getPlugin(), $form, $form_state, [
      'settings_container',
      'settings',
      $key,
      'formatter_settings',
      'plugin',
    ]);
    $form['settings_container']['settings'][$key] = [
      'pardot_key' => [
        '#type' => 'textfield',
        '#disabled' => $form_handler_id === '-1' ? FALSE : $row->getStatus(),
        '#title' => $this->t('Pardot Field Name'),
        '#description' => $this->t($description),
//        '#title_display' => 'invisible',
        '#required' => TRUE,
        '#default_value' => $row->getPardotKey(),
        '#prefix' => "<span class='form-item__label has-error'>$field_message</span>"
      ],
      'formatter_settings' => [
        '#disabled' => FALSE,
        '#type' => 'container',
        'plugin_type' => [
          '#type' => 'select',
          '#title' => $this->t('Formatter Selection'),
          '#access' => !($row instanceof MappedEmailField),
//          '#title_display' => 'invisible',
          '#default_value' => $row->getPluginType(),
          '#options' => $plugin_collection,
          '#required' => $row->isRequired(),
          '#ajax' => [
            'event' => 'change',
            'callback' => [$this, 'updateFieldMap'],
            'wrapper' => 'pardot-settings-wrapper',
            'trigger_as' => ['name' => "settings_$key"],
          ],
        ],
        'plugin' => $plugin_form,
        'submit' => $plugin_type_submit_button,
      ],
      'remove' => [
        '#disabled' => $form_handler_id === '-1' ? FALSE : $row->getStatus(),
        '#type' => 'submit',
        '#name' => 'remove_field_map_' . $key,
        '#value' => $this->t('Remove'),
        '#submit' => [[$this, 'removeFieldMapSubmit']],
        '#depth' => -4,
        '#limit_validation_errors' => [],
        '#ajax' => [
          'callback' => [$this, 'ajaxRemoveFieldMap'],
          'wrapper' => 'pardot-settings-wrapper',
        ],
      ],
    ];
  }

  /**
   * Return a submit button for on change field selection.
   *
   * @param int $key
   *   The row key.
   *
   * @return array
   *   The submit button for the row.
   */
  public function generatePluginTypeSubmitButton($key) {
    return [
      '#submit' => [[$this, 'pluginTypeChangeSubmit']],
      '#type' => 'submit',
      '#attributes' => ['class' => ['visually-hidden']],
      '#ajax' => [
        'callback' => [$this, 'ajaxChangeFieldMap'],
        'wrapper' => 'pardot-settings-wrapper',
        'effect' => 'fade',
      ],
      '#limit_validation_errors' => [],
      '#name' => "settings_$key",
      '#row_index' => $key,
    ];
  }

  /**
   * Given a trigger element and depth return the correct parents for the form area to reload on ajax.
   *
   * @param array $trigger_element
   *   The triggering element from the ajax call.
   * @param int $depth
   *   The depth to cut off the end to get to the root of the form.
   *
   * @return array
   *   Return the correct parents array for the ajax part of the form.
   */
  public function formParentsArray(array $trigger_element, int $depth) {
    $array_parents = array_slice($trigger_element['#parents'], 0, $depth);
    array_push($array_parents, 'settings_container');
    return $array_parents;
  }

  /**
   * This is used to update the dom of the table container when adding a row.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return array|mixed|null
   *   Just return the mapped field container.
   */
  public function ajaxAddFieldMap(array &$form, FormStateInterface $form_state) {
    $trigger_element = $form_state->getTriggeringElement();
    return NestedArray::getValue($form, $this->formParentsArray($trigger_element, -1));
  }

  /**
   * Add a new form field mapping row.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function addFieldMapSubmit(array &$form, FormStateInterface $form_state) {
    $pardot_form_map = $form_state->get('pardot_form_map');
    $pardot_form_map->appendMappedField();
    $form_state->set('pardot_form_map', $pardot_form_map);
    $form_state->setRebuild();
  }

  /**
   * This is used to update the dom of the table container when changing a row plugin.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return array|mixed|null
   *   Just return the mapped field container.
   */
  public function ajaxChangeFieldMap(array &$form, FormStateInterface $form_state) {
    $trigger_element = $form_state->getTriggeringElement();
    return NestedArray::getValue($form, $this->formParentsArray($trigger_element, -5));
  }

  /**
   * Set the mapping array on change.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function pluginTypeChangeSubmit(array &$form, FormStateInterface $form_state) {
    $trigger_element = $form_state->getTriggeringElement();
    $parents = array_slice($trigger_element['#array_parents'], 0, -1);
    $parents[] = 'plugin_type';
    $key = $trigger_element['#row_index'];
    $plugin_type = NestedArray::getValue($form, $parents)['#value'];
    $pardot_form_map = $form_state->get('pardot_form_map');
    // @todo prolly need to check if getMappedField is null.
    $mapped_field = $pardot_form_map->getMappedField($key);
    $mapped_field->setPluginType($plugin_type);
    if ($mapped_field && !($mapped_field instanceof MappedEmailField)) {
      if ($plugin_type) {
        $plugin = $this->pardotFormMapFormatterManagerPlugin->createInstance($plugin_type)
          ->setConfiguration([
            'entity_types' => $this->configuration['entity_types'],
            'entity_id' => $this->configuration['entity_type_id'] . ':' . $this->configuration['entity_id'],
          ]);
        $pardot_form_map->getMappedField($key)->setPlugin($plugin);
      }
      else {
        $pardot_form_map->getMappedField($key)->setPlugin([]);
      }
    }
    $form_state->set('pardot_form_map', $pardot_form_map);
    $form_state->setRebuild();
  }

  /**
   * This is used to update the dom of the table container when removing a row.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return array|mixed|null
   *   Just return the mapped field container.
   */
  public function ajaxRemoveFieldMap(array &$form, FormStateInterface $form_state) {
    $trigger_element = $form_state->getTriggeringElement();
    return NestedArray::getValue($form, $this->formParentsArray($trigger_element, -4));
  }

  /**
   * Remove specified row from mapping.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function removeFieldMapSubmit(array &$form, FormStateInterface $form_state) {
    $pardot_form_map = $form_state->get('pardot_form_map');
    $trigger_element = $form_state->getTriggeringElement();
    $parents = array_slice($trigger_element['#parents'], 0, -1);
    $map_index = array_pop($parents);
    $mappedRow = $pardot_form_map->getMappedField($map_index);
    if ($mappedRow && !($mappedRow instanceof MappedEmailField)) {
      $pardot_form_map->removeMappedField($map_index);
      $form_state->set('pardot_form_map', $pardot_form_map);
    }
    $form_state->setRebuild();
  }

  public function getPluginCollection() {
    $plugins = $this->pardotFormMapFormatterManagerPlugin->getDefinitions();
    $options = ['' => 'none'];
    foreach ($plugins as $plugin_name => $plugin_definition) {
      if (isset($plugin_definition['types']) && in_array($this->configuration['entity_type_id'], $plugin_definition['types'])) {
        $options[$plugin_name] = $plugin_definition['label'];
      }
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $array_parents = $form['#array_parents'];
    $values = $form_state->getCompleteFormState()->getValue($array_parents);
    $settings = $values['settings_container']['settings'];
    /** @var \Drupal\pardot\FormMap $pardot_form_map */
    $pardot_form_map = $form_state->get('pardot_form_map');
    $pardot_form_map->setPostUrl($values['post_url']);
    $pardot_form_map->setFormHandlerId((int) $values['form_handler_id']);
    if (is_array($settings)) {
      foreach ($settings as $key => $setting) {
        $plugin = PluginSubFormHelpers::pluginSubmit($pardot_form_map->getMappedField($key)
          ->getPlugin(), $form, $form_state, [
            'settings_container',
            'settings',
            $key,
            'formatter_settings',
            'plugin',
          ]);
        $plugin_configuration = $plugin ? $plugin->getConfiguration() : $plugin;
        $pardot_form_map->getMappedField($key)
          ->setPardotKey($setting['pardot_key'])
          ->setPluginType($setting['formatter_settings']['plugin_type'])
          ->setPlugin($plugin_configuration);
      }
    }
    $this->configuration['settings'] = $pardot_form_map->toArray();
  }

}
